#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include <string.h>

#include <fcntl.h>

#ifdef __CYGWIN__
typedef unsigned int socklen_t;
#endif

void usage(int argc, char* argv[])
{
  printf("Usage: %s <port>\n", argv[0]);
  printf("Example: %s 10025\n", argv[0]);
}

int
main (int argc, char *argv[])
{
  int ret;
  char buf[1024];
  socklen_t len;
  struct sockaddr_in my_addr, client_addr;
  int sock;
  int flags;

  if (argc < 2) {
	usage(argc, argv);
	exit(0);
  }

  sock = socket (AF_INET, SOCK_DGRAM, 0);
  if (sock < 0)
    {
      perror ("couldn't create socket");
      exit (1);
    }

#ifdef SET_BLOCKING
  printf("Setting BLOCK\n");
  flags = fcntl (sock, F_GETFL, 0);
  flags &= ~O_NONBLOCK;
  ret = fcntl (sock, F_SETFL, flags);
#else
  printf("Setting NONBLOCK\n");
  flags = fcntl (sock, F_GETFL, 0);
  flags &= O_NONBLOCK;
  ret = fcntl (sock, F_SETFL, flags);
#endif
  if (ret < 0)
   {
     perror ("trouble with fcntl blocking/nonblocking");
     exit (1);
  }


  bzero ((void*) &my_addr, sizeof (my_addr));
  my_addr.sin_family = AF_INET;
  my_addr.sin_addr.s_addr = htonl (INADDR_ANY);
  my_addr.sin_port = htons (atoi(argv[1]));
  if (bind (sock, (struct sockaddr *) &my_addr, sizeof (my_addr)) < 0)
    {
      perror ("couldn't bind to address");
      exit (1);
    }

  while (1)
    {
      len = sizeof (client_addr);
      printf("For recvfrom...\n");
      ret = recvfrom (sock, buf, 1024, 0, (struct sockaddr *) &client_addr, &len);
      printf ("recvfrom returned %d\n", ret);
      if (ret < 0)
{
  perror ("error getting data from socket");
  continue;
}

      /* Do something with the data. Echo it. */
      ret = sendto (sock, buf, ret, 0, (struct sockaddr *) &client_addr, len);
      if (ret < 0)
{
  perror ("error sending data");
  continue;
}

      printf ("Sent %d bytes to client at %x\n", ret, client_addr.sin_addr.s_addr);
    }

  return 0;
}


